@@ -1,6 +1,4 @@
-import gi
-gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk
+from minigalaxy.ui.gtk import Gtk
 
 css = '.test { border: 1px solid green; }'
 CSS_PROVIDER = Gtk.CssProvider()
