@@ -2,7 +2,8 @@ from zipfile import BadZipFile
 
 
 class Download:
-    def __init__(self, url, save_location, finish_func=None, progress_func=None, cancel_func=None, number=1, out_of_amount=1):
+    def __init__(self, url, save_location, finish_func=None, progress_func=None, cancel_func=None, number=1,
+                 out_of_amount=1):
         self.url = url
         self.save_location = save_location
         self.__finish_func = finish_func
@@ -15,15 +16,15 @@ class Download:
         if self.__progress_func:
             if self.out_of_amount > 1:
                 # Change the percentage based on which number we are
-                progress_start = 100/self.out_of_amount*(self.number-1)
-                percentage = progress_start + percentage/self.out_of_amount
+                progress_start = 100 / self.out_of_amount * (self.number - 1)
+                percentage = progress_start + percentage / self.out_of_amount
                 percentage = int(percentage)
             self.__progress_func(percentage)
 
     def finish(self):
         if self.__finish_func:
             try:
-                self.__finish_func()
+                self.__finish_func(self.save_location)
             except (FileNotFoundError, BadZipFile):
                 self.cancel()
 
