@@ -1,15 +1,15 @@
 import os
-import gi
+import locale
 
-gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk, Gdk, GdkPixbuf
 from minigalaxy.ui.login import Login
 from minigalaxy.ui.preferences import Preferences
 from minigalaxy.ui.about import About
 from minigalaxy.api import Api
 from minigalaxy.config import Config
 from minigalaxy.paths import UI_DIR, LOGO_IMAGE_PATH, THUMBNAIL_DIR
+from minigalaxy.translation import _
 from minigalaxy.ui.library import Library
+from minigalaxy.ui.gtk import Gtk, Gdk, GdkPixbuf
 
 
 @Gtk.Template.from_file(os.path.join(UI_DIR, "application.ui"))
@@ -26,20 +26,37 @@ class Window(Gtk.ApplicationWindow):
     window_library = Gtk.Template.Child()
 
     def __init__(self, name="Minigalaxy"):
+        current_locale = Config.get("locale")
+        default_locale = locale.getdefaultlocale()[0]
+        if current_locale == '':
+            locale.setlocale(locale.LC_ALL, (default_locale, 'UTF-8'))
+        else:
+            try:
+                locale.setlocale(locale.LC_ALL, (current_locale, 'UTF-8'))
+            except NameError:
+                locale.setlocale(locale.LC_ALL, (default_locale, 'UTF-8'))
         Gtk.ApplicationWindow.__init__(self, title=name)
+
         self.api = Api()
-        self.show_installed_only = False
         self.search_string = ""
         self.offline = False
 
         # Set library
         self.library = Library(self, self.api)
         self.window_library.add(self.library)
+        self.header_installed.set_active(Config.get("installed_filter"))
 
         # Set the icon
         icon = GdkPixbuf.Pixbuf.new_from_file(LOGO_IMAGE_PATH)
         self.set_default_icon_list([icon])
 
+        # Set theme
+        settings = Gtk.Settings.get_default()
+        if Config.get("use_dark_theme") is True:
+            settings.set_property("gtk-application-prefer-dark-theme", True)
+        else:
+            settings.set_property("gtk-application-prefer-dark-theme", False)
+
         # Show the window
         if Config.get("keep_window_maximized"):
             self.maximize()
@@ -50,13 +67,21 @@ class Window(Gtk.ApplicationWindow):
             os.makedirs(THUMBNAIL_DIR, mode=0o755)
 
         # Interact with the API
-        self.__authenticate()
-        self.HeaderBar.set_subtitle(self.api.get_user_info())
+        self.offline = not self.api.can_connect()
+        if not self.offline:
+            try:
+                self.__authenticate()
+                self.HeaderBar.set_subtitle(self.api.get_user_info())
+            except Exception as e:
+                print(e)
+                self.offline = True
         self.sync_library()
 
     @Gtk.Template.Callback("filter_library")
     def filter_library(self, switch, _=""):
         self.library.filter_library(switch)
+        if switch == self.header_installed:
+            Config.set("installed_filter", switch.get_active())
 
     @Gtk.Template.Callback("on_menu_preferences_clicked")
     def show_preferences(self, button):
@@ -72,18 +97,18 @@ class Window(Gtk.ApplicationWindow):
 
     @Gtk.Template.Callback("on_menu_logout_clicked")
     def logout(self, button):
-        # Unset everything which is specific to this user
-        self.HeaderBar.set_subtitle("")
-        Config.unset("username")
-        Config.unset("refresh_token")
-        self.hide()
-
-        # Show the login screen
-        self.__authenticate()
-        self.HeaderBar.set_subtitle(self.api.get_user_info())
-        self.sync_library()
-
-        self.show_all()
+        question = _("Are you sure you want to log out of GOG?")
+        if self.show_question(question):
+            # Unset everything which is specific to this user
+            self.HeaderBar.set_subtitle("")
+            Config.unset("username")
+            Config.unset("refresh_token")
+            self.hide()
+            # Show the login screen
+            self.__authenticate()
+            self.HeaderBar.set_subtitle(self.api.get_user_info())
+            self.sync_library()
+            self.show_all()
 
     @Gtk.Template.Callback("on_window_state_event")
     def on_window_state_event(self, widget, event):
