@@ -1,9 +1,14 @@
 import unittest
 import sys
+import os
 from unittest.mock import MagicMock, mock_open, patch
+
 m_config = MagicMock()
+m_paths = MagicMock()
 sys.modules['minigalaxy.config'] = m_config
-from minigalaxy.game import Game
+sys.modules['minigalaxy.paths'] = m_paths
+m_paths.CONFIG_GAMES_DIR = "/home/user/.config/minigalaxy/games/"
+from minigalaxy.game import Game  # noqa: E402
 
 
 class TestGame(unittest.TestCase):
@@ -84,16 +89,6 @@ class TestGame(unittest.TestCase):
         observed = game.is_update_available("82.8193.20.1", dlc_title="Neverwinter Nights: Wyvern Crown of Cormyr")
         self.assertEqual(expected, observed)
 
-    def test5_is_update_available(self):
-        game = Game("Version Test game")
-        game.load_minigalaxy_info_json = MagicMock()
-        game.load_minigalaxy_info_json.return_value = {'version': "91.8193.16", "dlcs": {}}
-        game.legacy_get_dlc_status = MagicMock()
-        game.legacy_get_dlc_status.return_value = "updatable"
-        expected = True
-        observed = game.is_update_available("82.8193.20.1", dlc_title="Neverwinter Nights: Wyvern Crown of Cormyr")
-        self.assertEqual(expected, observed)
-
     def test1_get_install_directory_name(self):
         game = Game("Get Install Directory Test1")
         expected = "Get Install Directory Test1"
@@ -138,50 +133,9 @@ en-US
             observed = game.fallback_read_installed_version()
         self.assertEqual(expected, observed)
 
-    @unittest.mock.patch('os.path.isfile')
-    def test1_legacy_get_dlc_status(self, mock_isfile):
-        mock_isfile.side_effect = [True]
-        json_content = '[{"Neverwinter Nights: Wyvern Crown of Cormyr": "not-installed", ' \
-                       '"Neverwinter Nights: Infinite Dungeons": "updatable", "Neverwinter Nights: Pirates of ' \
-                       'the Sword Coast": "installed"}, {}]'
-        with patch("builtins.open", mock_open(read_data=json_content)):
-            game = Game("Game Name test1")
-            game.read_installed_version = MagicMock()
-            game.installed_version = "1"
-            dlc_status = game.legacy_get_dlc_status("Neverwinter Nights: Wyvern Crown of Cormyr", "")
-        expected = "not-installed"
-        observed = dlc_status
-        self.assertEqual(expected, observed)
-
-    @unittest.mock.patch('os.path.isfile')
-    def test2_legacy_get_dlc_status(self, mock_isfile):
-        mock_isfile.side_effect = [True]
-        json_content = '[{"Neverwinter Nights: Wyvern Crown of Cormyr": "not-installed", ' \
-                       '"Neverwinter Nights: Infinite Dungeons": "updatable", "Neverwinter Nights: Pirates of ' \
-                       'the Sword Coast": "installed"}, {}]'
-        with patch("builtins.open", mock_open(read_data=json_content)):
-            game = Game("Game Name test2")
-            dlc_status = game.legacy_get_dlc_status("Neverwinter Nights: Infinite Dungeons", "")
-        expected = "updatable"
-        observed = dlc_status
-        self.assertEqual(expected, observed)
-
-    @unittest.mock.patch('os.path.isfile')
-    def test3_legacy_get_dlc_status(self, mock_isfile):
-        mock_isfile.side_effect = [False]
-        json_content = '[{"Neverwinter Nights: Wyvern Crown of Cormyr": "not-installed", ' \
-                       '"Neverwinter Nights: Infinite Dungeons": "updatable", "Neverwinter Nights: Pirates of ' \
-                       'the Sword Coast": "installed"}, {}]'
-        with patch("builtins.open", mock_open(read_data=json_content)):
-            game = Game("Game Name test2")
-            dlc_status = game.legacy_get_dlc_status("Neverwinter Nights: Infinite Dungeons", "")
-        expected = "not-installed"
-        observed = dlc_status
-        self.assertEqual(expected, observed)
-
-    @unittest.mock.patch('os.path.isfile')
-    def test1_set_info(self, mock_isfile):
-        mock_isfile.return_value = True
+    @unittest.mock.patch('os.path.exists')
+    def test1_set_info(self, mock_exists):
+        mock_exists.return_value = True
         json_content = '{"version": "gog-2"}'
         with patch("builtins.open", mock_open(read_data=json_content)) as m:
             game = Game("Game Name test2")
@@ -196,9 +150,10 @@ en-US
         observed = write_string
         self.assertEqual(expected, observed)
 
-    @unittest.mock.patch('os.path.isfile')
-    def test2_set_dlc_info(self, mock_isfile):
-        mock_isfile.return_value = False
+    @unittest.mock.patch('os.path.exists')
+    @unittest.mock.patch('os.makedirs')
+    def test2_set_dlc_info(self, mock_makedirs, mock_exists):
+        mock_exists.return_value = False
         dlc_name = "Neverwinter Nights: Wyvern Crown of Cormyr"
         with patch("builtins.open", mock_open()) as m:
             game = Game("Neverwinter Nights")
@@ -213,51 +168,6 @@ en-US
         observed = write_string
         self.assertEqual(expected, observed)
 
-    @unittest.mock.patch('os.path.isfile')
-    def test1_get_dlc_status_version(self, mock_isfile):
-        mock_isfile.side_effect = [True]
-        json_content = '[{"Neverwinter Nights: Wyvern Crown of Cormyr": "not-installed", ' \
-                       '"Neverwinter Nights: Infinite Dungeons": "installed", "Neverwinter Nights: Pirates of ' \
-                       'the Sword Coast": "installed"}, {"Neverwinter Nights: Wyvern Crown of Cormyr": ' \
-                       '"81.8193.16", "Neverwinter Nights: Infinite Dungeons": "81.8193.17", "Neverwinter Nights: ' \
-                       'Pirates of the Sword Coast": "81.8193.18"}] '
-        with patch("builtins.open", mock_open(read_data=json_content)):
-            game = Game("Game Name test2")
-            dlc_status = game.legacy_get_dlc_status("Neverwinter Nights: Infinite Dungeons", "81.8193.16")
-        expected = "updatable"
-        observed = dlc_status
-        self.assertEqual(expected, observed)
-
-    @unittest.mock.patch('os.path.isfile')
-    def test2_get_dlc_status_version(self, mock_isfile):
-        mock_isfile.side_effect = [True]
-        json_content = '[{"Neverwinter Nights: Wyvern Crown of Cormyr": "updatable", ' \
-                       '"Neverwinter Nights: Infinite Dungeons": "installed", "Neverwinter Nights: Pirates of ' \
-                       'the Sword Coast": "installed"}, {"Neverwinter Nights: Wyvern Crown of Cormyr": ' \
-                       '"81.8193.16", "Neverwinter Nights: Infinite Dungeons": "81.8193.17", "Neverwinter Nights: ' \
-                       'Pirates of the Sword Coast": "81.8193.18"}] '
-        with patch("builtins.open", mock_open(read_data=json_content)):
-            game = Game("Game Name test2")
-            dlc_status = game.legacy_get_dlc_status("Neverwinter Nights: Wyvern Crown of Cormyr", "")
-        expected = "updatable"
-        observed = dlc_status
-        self.assertEqual(expected, observed)
-
-    @unittest.mock.patch('os.path.isfile')
-    def test3_get_dlc_status_version(self, mock_isfile):
-        mock_isfile.side_effect = [True]
-        json_content = '[{"Neverwinter Nights: Wyvern Crown of Cormyr": "updatable", ' \
-                       '"Neverwinter Nights: Infinite Dungeons": "installed", "Neverwinter Nights: Pirates of ' \
-                       'the Sword Coast": "installed"}, {"Neverwinter Nights: Wyvern Crown of Cormyr": ' \
-                       '"81.8193.16", "Neverwinter Nights: Infinite Dungeons": "81.8193.17", "Neverwinter Nights: ' \
-                       'Pirates of the Sword Coast": "81.8193.18"}] '
-        with patch("builtins.open", mock_open(read_data=json_content)):
-            game = Game("Game Name test2")
-            dlc_status = game.legacy_get_dlc_status("Neverwinter Nights: Infinite Dungeons", "81.8193.17")
-        expected = "installed"
-        observed = dlc_status
-        self.assertEqual(expected, observed)
-
     def test_get_stripped_name(self):
         name_string = "Beneath A Steel Sky"
         game = Game(name_string)
@@ -287,7 +197,9 @@ en-US
         observed = jscon_dict
         self.assertEqual(expected, observed)
 
-    def test_save_minigalaxy_info_json(self):
+    @unittest.mock.patch("minigalaxy.config.Config")
+    @unittest.mock.patch('os.makedirs')
+    def test_save_minigalaxy_info_json(self, mock_makedirs, mock_config):
         json_dict = {"version": "gog-2"}
         with patch("builtins.open", mock_open()) as m:
             game = Game("Neverwinter Nights")
@@ -312,7 +224,7 @@ en-US
         self.assertEqual(exp, obs)
 
     @unittest.mock.patch('os.path.exists')
-    def test3_is_installed(self, mock_isfile):
+    def test2_is_installed(self, mock_isfile):
         mock_isfile.side_effect = [True]
         game = Game("Game Name Test", install_dir="Test Install Dir")
         game.load_minigalaxy_info_json = MagicMock()
@@ -322,7 +234,7 @@ en-US
         self.assertEqual(exp, obs)
 
     @unittest.mock.patch('os.path.exists')
-    def test4_is_installed(self, mock_isfile):
+    def test3_is_installed(self, mock_isfile):
         mock_isfile.side_effect = [True]
         game = Game("Game Name Test", install_dir="Test Install Dir")
         game.load_minigalaxy_info_json = MagicMock()
@@ -346,7 +258,7 @@ en-US
 
     @unittest.mock.patch('os.path.isfile')
     def test_get_dlc_info(self, mock_isfile):
-        mock_isfile.side_effect = [True]
+        mock_isfile.side_effect = [True, False]
         json_content = '{"dlcs": {"example_dlc" : {"example_key": "example_value"}}}'
         with patch("builtins.open", mock_open(read_data=json_content)):
             game = Game("Game Name test")
@@ -355,22 +267,106 @@ en-US
         observed = game_get_status
         self.assertEqual(expected, observed)
 
-    def test1_set_install_dir(self):
-        m_config.Config.get.return_value = "/home/user/GOG Games"
-        game = Game("Neverwinter Nights")
+    def test_set_install_dir(self):
+        install_directory = "/home/user/GOG Games"
+        install_game_name = "Neverwinter Nights"
+        m_config.Config.get.return_value = install_directory
+        game = Game(install_game_name)
         game.set_install_dir()
-        exp = "/home/user/GOG Games/Neverwinter Nights"
+        exp = os.path.join(install_directory, install_game_name)
         obs = game.install_dir
         self.assertEqual(exp, obs)
 
-    def test2_set_install_dir(self):
-        m_config.Config.get.return_value = "/home/user/GOG Games"
-        game = Game("Neverwinter Nights")
-        game.set_install_dir()
-        exp = "/home/user/GOG Games/Neverwinter Nights/minigalaxy-info.json"
-        obs = game.status_file_path
-        self.assertEqual(exp, obs)
+    @unittest.mock.patch('os.path.isfile')
+    def test1_get_info_legacy(self, mock_isfile):
+        mock_isfile.side_effect = [False, True]
+        json_content = '{"example_key": "example_value"}'
+        with patch("builtins.open", mock_open(read_data=json_content)):
+            game = Game("Game Name test")
+            game.set_info = MagicMock()
+            game_get_status = game.get_info("example_key")
+        expected = "example_value"
+        observed = game_get_status
+        self.assertEqual(expected, observed)
+
+    @unittest.mock.patch('os.path.isfile')
+    def test2_get_info_legacy(self, mock_isfile):
+        mock_isfile.side_effect = [True, True]
+        json_content = '{"example_key": "example_value"}'
+        with patch("builtins.open", mock_open(read_data=json_content)):
+            game = Game("Game Name test")
+            game.set_info = MagicMock()
+            game.load_minigalaxy_info_json = MagicMock()
+            game.load_minigalaxy_info_json.return_value = {}
+            game_get_status = game.get_info("example_key")
+        expected = "example_value"
+        observed = game_get_status
+        self.assertEqual(expected, observed)
+
+    @unittest.mock.patch('os.path.isfile')
+    def test3_get_info_legacy(self, mock_isfile):
+        mock_isfile.side_effect = [True, True]
+        json_content = '{"example_key": "example_value_legacy"}'
+        with patch("builtins.open", mock_open(read_data=json_content)):
+            game = Game("Game Name test")
+            game.load_minigalaxy_info_json = MagicMock()
+            game.load_minigalaxy_info_json.return_value = {"example_key": "example_value"}
+            game_get_status = game.get_info("example_key")
+        expected = "example_value"
+        observed = game_get_status
+        self.assertEqual(expected, observed)
+
+    @unittest.mock.patch('os.path.isfile')
+    def test1_get_dlc_info_legacy(self, mock_isfile):
+        mock_isfile.side_effect = [False, True]
+        json_content = '{"dlcs": {"example_dlc" : {"example_key": "example_value"}}}'
+        with patch("builtins.open", mock_open(read_data=json_content)):
+            game = Game("Game Name test")
+            game.set_dlc_info = MagicMock()
+            game_get_status = game.get_dlc_info("example_key", "example_dlc")
+        expected = "example_value"
+        observed = game_get_status
+        self.assertEqual(expected, observed)
+
+    @unittest.mock.patch('os.path.isfile')
+    def test2_get_dlc_info_legacy(self, mock_isfile):
+        mock_isfile.side_effect = [True, True]
+        json_content = '{"dlcs": {"example_dlc" : {"example_key": "example_value"}}}'
+        with patch("builtins.open", mock_open(read_data=json_content)):
+            game = Game("Game Name test")
+            game.set_dlc_info = MagicMock()
+            game.load_minigalaxy_info_json = MagicMock()
+            game.load_minigalaxy_info_json.return_value = {}
+            game_get_status = game.get_dlc_info("example_key", "example_dlc")
+        expected = "example_value"
+        observed = game_get_status
+        self.assertEqual(expected, observed)
+
+    @unittest.mock.patch('os.path.isfile')
+    def test3_get_dlc_info_legacy(self, mock_isfile):
+        mock_isfile.side_effect = [True, True]
+        json_content = '{"dlcs": {"example_dlc" : {"example_key": "example_value_legacy"}}}'
+        with patch("builtins.open", mock_open(read_data=json_content)):
+            game = Game("Game Name test")
+            game.load_minigalaxy_info_json = MagicMock()
+            game.load_minigalaxy_info_json.return_value = {"dlcs": {"example_dlc": {"example_key": "example_value"}}}
+            game_get_status = game.get_dlc_info("example_key", "example_dlc")
+        expected = "example_value"
+        observed = game_get_status
+        self.assertEqual(expected, observed)
+
+    def test1_get_status_file_path(self):
+        game = Game(name="Europa Universalis 2")
+        expected = "/home/user/.config/minigalaxy/games/Europa Universalis 2.json"
+        observed = game.get_status_file_path()
+        self.assertEqual(expected, observed)
+
+    def test2_get_status_file_path(self):
+        game = Game(name="Europa Universalis 2", install_dir="/home/user/GoG Games//Europa Universalis II")
+        expected = "/home/user/.config/minigalaxy/games/Europa Universalis II.json"
+        observed = game.get_status_file_path()
+        self.assertEqual(expected, observed)
 
 
 del sys.modules["minigalaxy.config"]
-del sys.modules["minigalaxy.game"]
+del sys.modules["minigalaxy.paths"]
